@@ -13,6 +13,8 @@ def _check_lengths(**kwargs):
     length = None
     for nn, xx in kwargs.items():
         if hasattr(xx, "__iter__"):
+            if hasattr(xx, 'shape') and len(xx.shape) == 0:
+                continue
             if length is None:
                 length = len(xx)
             else:
@@ -42,6 +44,8 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
                       scale_with_transverse_norm_emitt=None,
                       weight=None,
                       particle_class=Particles,
+                      co_search_settings=None,
+                      steps_r_matrix=None
                     ):
 
     """
@@ -121,6 +125,21 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
     if not isinstance(particle_ref._buffer.context, xo.ContextCpu):
         particle_ref = particle_ref.copy(_context=xo.ContextCpu())
 
+    # Move other input parameters to cpu if needed
+    # Generated by:
+    # for nn in 'x px y py zeta delta x_norm px_norm y_norm py_norm'.split():
+    #     print(f'{nn} = ({nn}.get() if hasattr({nn}, "get") else {nn})')
+    x = (x.get() if hasattr(x, "get") else x)
+    px = (px.get() if hasattr(px, "get") else px)
+    y = (y.get() if hasattr(y, "get") else y)
+    py = (py.get() if hasattr(py, "get") else py)
+    zeta = (zeta.get() if hasattr(zeta, "get") else zeta)
+    delta = (delta.get() if hasattr(delta, "get") else delta)
+    x_norm = (x_norm.get() if hasattr(x_norm, "get") else x_norm)
+    px_norm = (px_norm.get() if hasattr(px_norm, "get") else px_norm)
+    y_norm = (y_norm.get() if hasattr(y_norm, "get") else y_norm)
+    py_norm = (py_norm.get() if hasattr(py_norm, "get") else py_norm)
+
     if tracker is not None and tracker.iscollective:
         logger.warning('Ignoring collective elements in particles generation.')
         tracker = tracker._supertracker
@@ -172,7 +191,8 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
             particle_on_co = tracker.find_closed_orbit(
                 particle_co_guess=Particles(
                     x=0, px=0, y=0, py=0, zeta=0, delta=0.,
-                    **ref_dict))
+                    **ref_dict), 
+                co_search_settings=co_search_settings)
         else:
             assert particle_on_co._capacity == 1
 
@@ -181,7 +201,7 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
 
         if R_matrix is None:
             R_matrix = tracker.compute_one_turn_matrix_finite_differences(
-                particle_on_co=particle_on_co)
+                particle_on_co=particle_on_co, steps_r_matrix=steps_r_matrix)
 
         num_particles = _check_lengths(num_particles=num_particles,
             zeta=zeta, delta=delta, x_norm=x_norm, px_norm=px_norm,
